@@ -22,6 +22,7 @@ class ChatWidget;
 class Shot;
 class AudioPlayer;
 class Protocol;
+class BattleShipsConfiguration;
 
 class Controller : public QObject
 {
@@ -33,7 +34,7 @@ Q_OBJECT
     int m_ready;
     AudioPlayer* m_player;
     bool m_has_ai;
-    bool m_allow_adjacent_ships;
+    BattleShipsConfiguration mBattleShipsConfiguration;
 
     void notify(Sea::Player player, const Coord& c, const HitInfo& info);
     void setupEntity(Entity*);
@@ -44,7 +45,7 @@ Q_OBJECT
 
     friend class Shot;
 public:
-    explicit Controller(QObject* parent, AudioPlayer* audioPlayer = 0, const bool allow_adjacent_ships = false);
+    explicit Controller(QObject* parent, AudioPlayer* audioPlayer = 0, const BattleShipsConfiguration& battleConfiguration = BattleShipsConfiguration::defaultSingleShipsConfiguration(true));
 
     PlayerEntity* createPlayer(Sea::Player player, SeaView* view,
                                ChatWidget* chat, const QString& nick);
@@ -56,14 +57,22 @@ public:
     Sea::Player turn() const;
     bool hasAI() const;
     inline Sea* getSea() const { return m_sea; }
+    void setBattleShipsConfiguration(const BattleShipsConfiguration& battleConfiguration);
+    inline BattleShipsConfiguration& getBattleShipsConfiguration() { return mBattleShipsConfiguration; }
 public slots:
     void shoot(int player, const Coord& c);
     void ready(int player);
+    void shipsPlaced(int player);
     void receivedChat(const QString& text);
     void nick(int player, const QString& nick);
+    void notifyRestartPlacingShips(Sea::Player player);
+    void placing(bool ask = false);
+    void restart(bool ask = false);
 signals:
     void gameOver(Sea::Player);
     void restartRequested();
+    void startPlacingShips(int player);
+    void restartPlacingShips(Sea::Player player); // in case it is impossible to finish with the current board
     void compatibility(int);
     void nickChanged(int, const QString&);
     void turnChanged(int);
